/*
 First rev of firmware for Makerboard for TMG soldering course.  
 
 On power-up will make a little light show and then go to sleep.  Will use the reset button to wake up from sleep.  Once the basics are in the bag might implement EEPROM and have a few different routines and index them with a state machine, storing the state variable in EEPROM and incrementing on wake.  
 
 

 
 Have 5 LED's in the following Format:
 
 
            | LED5
 
 
   \ LED4       LED2 /
 
 
   _ LED1       LED3 _
 
 LED1 = PB0  OC0A
 LED2 = PB1  OC0B
 LED3 = PB2
 LED4 = PB3
 LED5 = PB4
 
 
 
 */


#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "Makerboard.h"
#include "Timers.h"
#include <avr/sleep.h>
#include <avr/EEPROM.h>


//will run the PWM values on Timer0 from here.
void updateT0Counters(uint8_t newVal){
    OCR0A = newVal;
    OCR0B = newVal;
}





uint8_t caseIndexer = 0;    //will pull this variable out of EEPROM and use it to index the state machine for which routine to run


int main(void) {
    caseIndexer = eeprom_read_byte((uint8_t*)10);
    if (caseIndexer > 4){
        caseIndexer = 0;
    }
   // caseIndexer = 4;
    DDRB |= 0x1F;   //PB0 - PB4 are outputs for the LEDs
  //  Timer1_init();  //initialize the millisecond timer....On second thought why do I need a millisecond timer?
    set_sleep_mode(SLEEP_MODE_PWR_DOWN);
    sei();
  

	while (1) {
        
        switch (caseIndexer){
            case 0:
                firstRun();
                break;
            case 1:
                secondRun();
                break;
            case 2:
                thirdRun();
                break;
            case 3:
                fourthRun();
                break;
            case 4:
                fifthrun();
                break;
        }
        caseIndexer ++;
        eeprom_write_byte((uint8_t*)10, caseIndexer);
        sleep_mode();
        asm("nop");
        asm("nop");
   

     
	}
	return 0; // never reached
}




void firstRun(void){
    uint8_t cycles = 0;
    while(cycles < 6){
        LED3ON
        _delay_ms(250);
        LED1OFF
        LED2ON
        _delay_ms(250);
        LED3OFF
        LED5ON
        _delay_ms(250);
        LED2OFF
        LED4ON
        _delay_ms(250);
        LED5OFF
        LED1ON
        _delay_ms(250);
        LED4OFF
        cycles ++;
    }
    PORTB = 0;
    
}


void secondRun(void){
    LED1ON
    _delay_ms(150);
    PORTB = 0;
    LED4ON
    _delay_ms(150);
    PORTB = 0;
    LED5ON
    _delay_ms(150);
    PORTB = 0;
    LED2ON
    _delay_ms(150);
    PORTB = 0;
    LED3ON
    _delay_ms(150);
    PORTB = 0;
}

void thirdRun(void){
    uint8_t cycles = 0;
    while(cycles < 4){
    LED1ON
    _delay_ms(250);
    LED3OFF
    LED4ON
    _delay_ms(250);
    LED1OFF
    LED5ON
    _delay_ms(250);
    LED4OFF
    LED2ON
    _delay_ms(250);
    LED5OFF
    LED3ON
    _delay_ms(250);
    LED2OFF
        cycles ++;
    }
    PORTB = 0;
    
}


void fourthRun(void){
    uint8_t cntr = 0;
    int8_t chngby = 1;
    uint8_t cycles = 0;
    Timer0_init();  //enable the timer
    
    while(cycles < 3){
    cntr += chngby;
    updateT0Counters(cntr);
    _delay_ms(5);
    if((cntr == 255) || (cntr == 0)){
        _delay_ms(175);
        chngby = -chngby;
        if (cntr == 0){
            cycles ++;
            LED4ON
            LED5ON
            LED3ON
            _delay_ms(9);
            LED4OFF
            LED5OFF
            LED3OFF
            
        }
        
    }
        
    }
    TCCR0A = 0; //this will release the COM0Ax pins so we all turn off.
}



void fifthrun(void){
    uint8_t cycles = 0;
    while (cycles < 6){
        LED1ON
        LED3ON
        LED5OFF
        _delay_ms(255);
        LED1OFF
        LED3OFF
        LED2ON
        LED4ON
        _delay_ms(255);
        LED2OFF
        LED4OFF
        LED5ON
        _delay_ms(255);
        cycles ++;
    }
    PORTB = 0;
}
